#include <stdio.h>
#include <math.h>
#include <inttypes.h>

#define SAMPLE_RATE	48000
#define TONE_FREQ	1000

// one test duration (500 ms) in samples (not TONE_SAMPLES)
#define TEST_DURATION	(SAMPLE_RATE / 2)

// number of samples in one tone
#define TONE_SAMPLES	(SAMPLE_RATE / TONE_FREQ)

// initial tone length to generate, in TONE_SAMPLES
// 100 ms
#define INITIAL_LENGTH	(TONE_FREQ / 10)

// generates TONE_SAMPLES of frequency if channel == 0 or 1
// generates TONE_SAMPLES of silence if channel == 2
static void generate_chunk(int channel)
{
	unsigned i;
	for (i = 0; i < TONE_SAMPLES; ++i)
	{
		const double a = (double) i * 2 * M_PI / TONE_SAMPLES;
		const double s = sin(a);
		const int16_t sample = s * 32767;
		const int16_t silence = 0;

		// note the endianness matters here
		fwrite((channel == 0) ? &sample : &silence, sizeof(sample), 1, stdout);
		fwrite((channel == 1) ? &sample : &silence, sizeof(sample), 1, stdout);
	}
}

static void generate(int channel, unsigned length)
{
	const unsigned len_samples = length * TONE_SAMPLES;
	unsigned j;

	for (j = 0; j < TEST_DURATION / len_samples * 2; ++j)
	{
		unsigned i;

		for (i = 0; i < length; ++i)
			generate_chunk(channel);

		for (i = 0; i < length; ++i)
			generate_chunk(2);
	}
}

int main(void)
{
	unsigned length;

	for (length = INITIAL_LENGTH; length; length /= 2)
	{
		// left channel
		generate(0, length);

		// right channel
		generate(1, length);
	}

	return 0;
}
